// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Columna

#include "columna.h"
#include "tablero.h"

void Columna::Iniciar(Tablero &t, int c) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(i, c);
}

bool Columna::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Columna::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

void Columna::CribaMenosCaja(int f) {
    for(int i=0; i<9; i++) if(i/3 != f) cas[i]->MarcaNoPosible();
}

Unico Columna::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

int Columna::UnicoPosible() {
    int cuenta=0;
    int pos = -1;
    for(int i=0; i<9; i++) {
        if(cas[i]->Posible()) {
            cuenta++;
            pos = i;
        }
    }
    if(cuenta == 1) return pos;
    return -1;
}

void Columna::DesmarcaLapiz(int v) {
    for(int f=0; f<9; f++) cas[f]->Lapiz().EliminaMarca(v);
}

void Columna::DesmarcaLapizMenosCaja(int k, int v) {
    for(int i=0; i<9; i++)
        if(i/3 != k) cas[i]->Lapiz().EliminaMarca(v);
}

Unico Columna::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int c=0; c<9; c++) {
            if(!cas[c]->Valor()) {
                if(cas[c]->Lapiz().ContieneMarca(v)) {
                    u = Unico(c, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Columna::BuscaAla1MenosCaja(Casilla *pivote, int f1) {
    int k = pivote->Caja();

    for(int f=f1; f<9; f++) {
        if(cas[f]->Caja() != k && cas[f]->Lapiz().CuentaMarcas() == 2) { // Distinta caja con dos marcas de lpiz
            if((cas[f]->Lapiz() + pivote->Lapiz()).CuentaMarcas() == 3) { // La casilla tiene un candidato en comn con el pivote
                return cas[f];
            }
        }
    }
    return NULL;
}

Casilla *Columna::BuscaAla2MenosCaja(Casilla *pivote, Casilla *ala1) {
    int k = pivote->Caja();
    MarcasLapiz nocomunes = (pivote->Lapiz() + ala1->Lapiz()) - (pivote->Lapiz() ^ ala1->Lapiz());

    for(int f=0; f<9; f++) {
        if(cas[f]->Caja() != k && cas[f]->Lapiz() == nocomunes) { // Distinta caja con las dos marcas de lpiz comunes
            return cas[f];
        }
    }
    return NULL;
}

Pareja Columna::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int f=0; f<9; f++) {
        if(cas[f]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[f]);
    }
    if(cuenta==2) return par;
    return Pareja();
}
