// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Fila

#include "fila.h"
#include "tablero.h"

void Fila::Iniciar(Tablero &t, int f) {
    for(int i = 0; i < 9; i++) cas[i] = &t.GetCasilla(f, i);
}

bool Fila::Presente(int n) {
    for(int i=0; i<9; i++) if(cas[i]->Valor() == n) return true;
    return false;
}

void Fila::Criba() {
    for(int i=0; i<9; i++) cas[i]->MarcaNoPosible();
}

void Fila::CribaMenosCaja(int c) {
    for(int i=0; i<9; i++) if(i/3 != c) cas[i]->MarcaNoPosible();
}

Unico Fila::CandidatoUnico() {
    bool valor[9] = {false, false, false, false, false, false, false, false, false};
    int cuenta = 0;
    int i, c;

    // Contar valores, si son 8 devolver valor que falta:
    for(i=0; i<9; i++) {
        if(cas[i]->Valor()) {
            cuenta++;
            valor[cas[i]->Valor()-1] = true;
        } else
            c=i;
    }
    if(cuenta == 8) {
        i=0;
        while(valor[i++]);
        return Unico(c, i);
    }
    return Unico(-1,0);
}

int Fila::UnicoPosible() {
    int cuenta=0;
    int pos = -1;
    for(int i=0; i<9; i++) {
        if(cas[i]->Posible()) {
            cuenta++;
            pos = i;
        }
    }
    if(cuenta == 1) return pos;
    return -1;
}

void Fila::DesmarcaLapiz(int v) {
    for(int c=0; c<9; c++) cas[c]->Lapiz().EliminaMarca(v);
}

void Fila::DesmarcaLapizMenosCaja(int k, int v) {
    for(int i=0; i<9; i++)
        if(i/3 != k) cas[i]->Lapiz().EliminaMarca(v);
}

Unico Fila::UnicaCasillaConCandidato() {
    Unico u(-1, 0);
    int cuenta;

    for(int v=1; v<=9; v++) {
        cuenta = 0;
        for(int c=0; c<9; c++) {
            if(!cas[c]->Valor()) {
                if(cas[c]->Lapiz().ContieneMarca(v)) {
                    u = Unico(c, v);
                    cuenta++;
                }
            }
        }
        if(cuenta==1) return u;
    }
    return Unico(-1, 0);
}

Casilla *Fila::BuscaAla1MenosCaja(Casilla *pivote, int c1) {
    int k = pivote->Caja();

    for(int c=c1; c<9; c++) {
        if(cas[c]->Caja() != k && cas[c]->Lapiz().CuentaMarcas() == 2) { // Distinta caja con dos marcas de lpiz
            if((cas[c]->Lapiz() + pivote->Lapiz()).CuentaMarcas() == 3) { // La casilla tiene un candidato en comn con el pivote
                return cas[c];
            }
        }
    }
    return NULL;
}

Pareja Fila::DobleCandidato(int v) {
    int cuenta=0;
    Pareja par;

    for(int c=0; c<9; c++) {
        if(cas[c]->Lapiz().ContieneMarca(v)) par.Asigna(cuenta++, cas[c]);
    }
    if(cuenta==2) return par;
    return Pareja();
}
