// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net

#include <stdio.h>
#include <windows.h>
#include "lista.h"
#include "casilla.h"
#include "tablero.h"

char tablero[] = {
    "100009008000000000070430106030067000056000280000290060603058010000000000200900004"
//    "280000164009000000610704800000000900000097002050100000100870093320000000000003400"
//    "500200040000603000030009007003007000007008000600000020080000003000400600000100500"
};

// Funcin para mostrar texto en pantalla de consola
void Texto(int f, int c, const char *texto, WORD color) {
    COORD pos;
    DWORD escritos;

    pos.X = c;
    pos.Y = f;
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), color);
    SetConsoleCursorPosition(GetStdHandle(STD_OUTPUT_HANDLE), pos);
    WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), texto, strlen(texto), &escritos, NULL);
}

void LogPantalla(char *cad) {
    HANDLE hOut;
    CONSOLE_SCREEN_BUFFER_INFO csbInfo;
    SMALL_RECT sr_ScrollRect = { 0, 42, 50, 52 };// 42, 0, 52, 50
    SMALL_RECT sr_ClipRect = sr_ScrollRect;
    CHAR_INFO chInfo;
    COORD pos = {0, 41};

    chInfo.Attributes = 15;
    chInfo.Char.AsciiChar = ' ';
    hOut = GetStdHandle(STD_OUTPUT_HANDLE);
    GetConsoleScreenBufferInfo(hOut, &csbInfo);
    ScrollConsoleScreenBuffer(hOut, &sr_ScrollRect, &sr_ClipRect, pos, &chInfo);
    Texto(51, 0, cad, 15);
}

// Pausa hasta que se pulse intro
void Pausa() {
    INPUT_RECORD cad[100];
    DWORD leido;

    ReadConsoleInput(GetStdHandle(STD_INPUT_HANDLE), cad, 100, &leido);
    ReadConsoleInput(GetStdHandle(STD_INPUT_HANDLE), cad, 100, &leido);
}

int main()
{
    bool repetir;
    bool insistir;
    char cad[100];

    Tablero T(tablero);
    T.Mostrar();

    T.Soluciones();
    // Si la solucin no es nica, no intentar resolver
    if(T.Cuenta() != 1) {
        if(T.Cuenta() == 0)
            Texto(40, 1, "El sudoku no tiene solucin", 15);
        else
            Texto(40, 1, "El sudoku tiene varias soluciones", 15);
        return 0;
    }

    do {
        repetir=false;
        for(int v=1; v<=9; v++) {
            sprintf(cad, "Criba con valor %d", v);
            Texto(1, 40, cad, 7);
            do {
                T.Criba(v);
                insistir = T.ProcesarCriba(v);
                repetir |= insistir;
                T.Mostrar();
//                Pausa();
            } while(insistir);
        }
        if(!repetir && T.CandidatoUnico()) {
            repetir = true;
            Texto(2,40,"ProcesarCandidatoUnico", 7);
            T.Mostrar();
//            Pausa();
        }
        if(!repetir) {
            for(int v=1; v<=9; v++) {
                sprintf(cad, "Criba con valor %d", v);
                Texto(1, 40, cad, 7);
                T.Criba(v);
                repetir |= T.ProcesarCribaSolitarios(v);
                if(repetir) {
                    Texto(3,40,"ProcesarCribaSolitarios", 7);
                    T.Mostrar();
 //                   Pausa();
                }
            }
        }
        if(!repetir) {
            T.BuscaParesDesnudos();
            if(T.ProcesaParesDesnudos()) {
                repetir = true;
                Texto(4,40,"ProcesaParesDesnudos", 7);
                T.Mostrar();
//                Pausa();
            }
        }
    } while(repetir);

    T.LimpiaCriba();
    T.LimpiaResaltes();

    // FASE 2:
    T.Log("Fase 2");
    T.PonerMarcasLapiz();
    T.Mostrar();
//    Pausa();
    T.VerificarFinales();
    T.Mostrar();
//    Pausa();

    do {
        repetir=false;
        T.BuscaParesDesnudos();
        if(T.ProcesaParesDesnudosFase2()) {
            repetir =true;
            Texto(7,40,"ProcesaParesDesnudos", 7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        T.BuscaPunteros();
        Texto(8,40,"BuscaPunteros",7);
        if(!repetir && T.ProcesaPunteros()) {
            repetir =true;
            Texto(9,40,"ProcesaPunteros",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(2)) {
            repetir =true;
            Texto(10,40,"Grupos de dos",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(3)) {
            repetir =true;
            Texto(10,40,"Grupos de tres",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaGrafo(4)) {
            repetir =true;
            Texto(11,40,"Grupos de cuatro",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXYWing()) {
            repetir = true;
            Texto(12,40,"Procesa XY-Wing",7);
            T.VerificarFinales();
            T.Mostrar();
//            Pausa();
        }
        if(!repetir && T.ProcesaXWing()) {
            repetir = true;
            Texto(12,40,"Procesa X-Wing",7);
            T.Mostrar();
            Pausa();
            T.VerificarFinales();
            T.Mostrar();
            Pausa();
        }
    } while(repetir);

    T.LimpiaCriba();
    T.LimpiaResaltes();
    T.Mostrar();
    return 0;
}
