// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Definicin de la clase Tablero

#include <stdio.h>
#include <windows.h>
#include "tablero.h"

extern void Texto(int f, int c, const char *texto, WORD color);
extern void LogPantalla(char *cad);

Tablero::Tablero(char* t) : cuenta(0) {
    int f, c;

    for(f=0; f < 9; f++)
        for(c=0; c < 9; c++) {
            casilla[f][c] = Casilla(f, c, t[f*9+c]-'0', t[f*9+c] != '0');
        }
    for(f=0; f < 9; f++) fila[f].Iniciar(*this, f);
    for(c=0; c < 9; c++) columna[c].Iniciar(*this, c);
    for(c=0; c < 9; c++) caja[c].Iniciar(*this, c);
    flog = std::ofstream("solucion.log", std::ios::out);
}

void Tablero::Log(char *cad) {
    flog << cad << std::endl;
    LogPantalla(cad);
}

bool Tablero::Valido(int f, int c, int v) {
    return !fila[f].Presente(v) && !columna[c].Presente(v) && !caja[(f/3)*3+(c/3)].Presente(v);
}

bool Tablero::Soluciones() {
    int f, c, n;
    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(casilla[f][c].Valor() == 0) {
                for(n=1; n<=9; n++) {
                    if(Valido(f, c, n)) {
                        casilla[f][c].Asignar(n);
                        if(Soluciones()) return true;
                        else casilla[f][c].Asignar(0);
                    }
                }
                return false;
            }
        }
    }
    cuenta++;
    return false;
}

void Tablero::Asignar(int f, int c, int v) {
    casilla[f][c].Asignar(v);
    casilla[f][c].Resaltar();
    // Eliminar marcas de lapiz en fila, columna y caja:
    int k = (f/3)*3 + (c/3);
    for(int i=0; i<9; i++) {
        if(fila[f].Cas(i)->Lapiz().EliminaMarca(v)) fila[f].Cas(i)->Resaltar();
        if(columna[c].Cas(i)->Lapiz().EliminaMarca(v)) columna[c].Cas(i)->Resaltar();
        if(caja[k].Cas(i)->Lapiz().EliminaMarca(v)) caja[k].Cas(i)->Resaltar();
    }
}

void Tablero::LimpiaResaltes() {
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].Resaltar(false);
            casilla[f][c].Amarillo(false);
            casilla[f][c].Naranja(false);
        }
    }
}

void Tablero::LimpiaCriba() {
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].MarcaPosible();
        }
    }
}

void Tablero::Criba(int v) {
    int ncaja=0;

    LimpiaCriba();
    LimpiaResaltes();
    // Aplicar criba a valores finales:
    for(int i=0; i<9; i++) {
        if(fila[i].Presente(v)) fila[i].Criba();
        if(columna[i].Presente(v)) columna[i].Criba();
        if(caja[i].Presente(v)) caja[i].Criba();
    }
    // Eliminar de la criba el resto de casillas con valor final
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Valor()) casilla[f][c].MarcaPosible(false);
        }
    }
    // Aplicar criba a parejas y trios punteros de cajas:
    // No desmarcar como posibles las casillas miembros de parejas y trios punteros:
    for(int f=0; f<3; f++) { // Fila de cajas
        for(int c=0; c<3; c++) { // Columna de cajas
            for(int i=0; i<3; i++) { // Fila de punteros
                if(caja[ncaja].PunteroFila(i, v)) fila[f*3+i].CribaMenosCaja(c);
            }
            for(int i=0; i<3; i++) { // Columna de punteros
                if(caja[ncaja].PunteroColumna(i, v)) columna[c*3+i].CribaMenosCaja(f);
            }
            ncaja++;
        }
    }
}

bool Tablero::ProcesarCriba(int v) {
    bool retval = false;
    int p, s, t;
    int f, c;
    char cad[100];

    for(int i=0; i<9; i++) { // Verificar cada caja
        switch(caja[i].CuentaPosibles()) {
        case 1:
            // Final:
            p = caja[i].PrimeroPosible();
            f = caja[i].Cas(p)->Fila();
            c = caja[i].Cas(p)->Columna();
            Asignar(f, c, v);
            sprintf(cad, "Valor final %d en %c%c", v, 'A'+f, '1'+c);
            Log(cad);
            retval=true;
            break;
        case 2:
            // pareja puntero:
            p = caja[i].PrimeroPosible();
            s = caja[i].SegundoPosible(p);
            // Aadir marcas de lpiz y marcarlas en el tablero
            if(caja[i].Cas(p)->Lapiz().AsignaMarca(v))
                caja[i].Cas(p)->Resaltar();
            if(caja[i].Cas(s)->Lapiz().AsignaMarca(v))
                caja[i].Cas(s)->Resaltar();
            // Verificar si p y s estn alineados y guardar pareja puntero si lo estn
            if(caja[i].Cas(p)->Fila() == caja[i].Cas(s)->Fila()) {
                if(caja[i].AsignaPunteroFila(caja[i].Cas(p)->Fila()%3, v, 2)) {
                    sprintf(cad, "Par puntero caja %d fila %c, valor %d", 1+i, 'A'+caja[i].Cas(p)->Fila(), v);
                    Log(cad);
                    retval=true;
                }
            }
            if(caja[i].Cas(p)->Columna() == caja[i].Cas(s)->Columna()) {
                if(caja[i].AsignaPunteroColumna(caja[i].Cas(p)->Columna()%3, v, 2)) {
                    sprintf(cad, "Par puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            break;
        case 3:
            // trio puntero:
            p = caja[i].PrimeroPosible();
            s = caja[i].SegundoPosible(p);
            t = caja[i].TerceroPosible(s);
            // Verificar si p, s y t estn alineados y guardar trie puntero si lo estn
            if(caja[i].Cas(p)->Fila() == caja[i].Cas(s)->Fila() && caja[i].Cas(p)->Fila() == caja[i].Cas(t)->Fila()) {
                if(caja[i].AsignaPunteroFila(caja[i].Cas(p)->Fila()%3, v, 3)) {
                    sprintf(cad, "Trio puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            if(caja[i].Cas(p)->Columna() == caja[i].Cas(s)->Columna() && caja[i].Cas(p)->Columna() == caja[i].Cas(t)->Columna()) {
                if(caja[i].AsignaPunteroColumna(caja[i].Cas(p)->Columna()%3, v, 3)) {
                    sprintf(cad, "Trio puntero caja %d columna %c, valor %d", 1+i, '1'+caja[i].Cas(p)->Columna(), v);
                    Log(cad);
                    retval=true;
                }
            }
            break;
        }
    }
    return retval;
}

bool Tablero::CandidatoUnico() {
    bool retval = false;
    Unico u(0,0);
    char cad[100];

    // Buscar candidatos nicos en filas, columnas y cajas:
    for(int f=0; f<9; f++) {
        u = fila[f].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(f, u.Casilla(), u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+f, '1'+u.Casilla());
            Log(cad);
            retval = true;
        }
    }

    for(int c=0; c<9; c++) {
        u = columna[c].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(u.Casilla(), c, u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+u.Casilla(), '1'+c);
            Log(cad);
            retval = true;
        }
    }

    for(int i=0; i<9; i++) {
        u = caja[i].CandidatoUnico();
        if(u.Casilla() != -1) {
            Asignar(caja[i].Cas(u.Casilla())->Fila(), caja[i].Cas(u.Casilla())->Columna(), u.Valor());
            sprintf(cad, "Candidato unico: Valor final %d en %c%c", u.Valor(), 'A'+caja[i].Cas(u.Casilla())->Fila(), '1'+caja[i].Cas(u.Casilla())->Columna());
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::ProcesarCribaSolitarios(int v) {
    bool retval = false;
    int fil, col;
    char cad[100];

    for(int i=0; i<9; i++) { // Verificar cada fila
        col = fila[i].UnicoPosible();
        if(col != -1) {
            Asignar(i, col, v);
            sprintf(cad, "Candidato solitario: Valor final %d en %c%c", v, 'A'+i, '1'+col);
            Log(cad);
            retval = true;
        }
    }

    for(int i=0; i<9; i++) { // Verificar cada columna
        fil = columna[i].UnicoPosible();
        if(fil != -1) {
            Asignar(fil, i, v);
            retval = true;
            sprintf(cad, "Candidato solitario: Valor final %d en %c%c", v, 'A'+fil, '1'+i);
            Log(cad);
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosFila(int f) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        // Solo buscamos en casillas vacas con dos marcas de lpiz.
        if(casilla[f][i].Valor() == 0 && casilla[f][i].Lapiz().CuentaMarcas() == 2) {
            cuenta = coincidencia = 0;
            des.AsignarCas(0, &casilla[f][i]);
            for(int v=1; v<=9; v++) {
                if(casilla[f][i].Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int k=i+1; k<9; k++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(casilla[f][k].Valor() == 0 && casilla[f][k].Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, &casilla[f][k]);
                        for(int v=1; v<=9; v++) {
                            if(casilla[f][k].Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            i++;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosColumna(int c) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        if(casilla[i][c].Valor() == 0 && casilla[i][c].Lapiz().CuentaMarcas() == 2) { // Solo buscamos en casillas vacas
            cuenta = coincidencia = 0;
            des.AsignarCas(0, &casilla[i][c]);
            for(int v=1; v<=9; v++) {
                if(casilla[i][c].Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int k=i+1; k<9; k++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(casilla[k][c].Valor() == 0 && casilla[k][c].Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, &casilla[k][c]);
                        for(int v=1; v<=9; v++) {
                            if(casilla[k][c].Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            i++;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                        break;
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudosCaja(int k) {
    int cuenta, coincidencia;
    Desnudo des(2);
    bool retval = false;
    char cad[100];

    for(int i=0; i<8; i++) {
        if(caja[k].Cas(i)->Valor() == 0 && caja[k].Cas(i)->Lapiz().CuentaMarcas() == 2) { // Solo buscamos en casillas vacas
            cuenta = coincidencia = 0;
            des.AsignarCas(0, caja[k].Cas(i));
            for(int v=1; v<=9; v++) {
                if(caja[k].Cas(i)->Lapiz().ContieneMarca(v)) {
                    des.AsignarVal(cuenta, v);
                    cuenta++;
                }
            }
            if(cuenta==2) { // con dos valores posibles
                for(int p=i+1; p<9; p++) {
                    // Buscamos en el resto de casillas vacas con dos valores posibles
                    if(caja[k].Cas(p)->Valor() == 0 && caja[k].Cas(p)->Lapiz().CuentaMarcas() == 2) {
                        cuenta = 0;
                        des.AsignarCas(1, caja[k].Cas(p));
                        for(int v=1; v<=9; v++) {
                            if(caja[k].Cas(p)->Lapiz().ContieneMarca(v) && des.Val(cuenta) == v) {
                                coincidencia++;
                                cuenta++;
                            }
                        }
                    }
                    if(cuenta==2 && coincidencia==2) {
                        retval = true;
                        if(desnudo.Insertar(des)) {
                            sprintf(cad, "Par desnudo: %c%c, %c%c valores %d, %d",
                                'A'+des.Cas(0)->Fila(), '1'+des.Cas(0)->Columna(),
                                'A'+des.Cas(1)->Fila(), '1'+des.Cas(1)->Columna(),
                                des.Val(0), des.Val(1));
                            Log(cad);
                            retval = true;
                        } else {
                            desnudo.ValorActual().MarcarParaNoBorrar();
                        }
                    }
                }
            }
        }
    }
    return retval;
}

bool Tablero::BuscaParesDesnudos() {
    bool retval=false;

    // Marcar todos los elementos para borrar:
    for(ListaDesnudo()->Primero(); ListaDesnudo()->Actual(); ListaDesnudo()->Siguiente())
        ListaDesnudo()->ValorActual().MarcarParaBorrar();

    for(int f=0; f<9; f++) {
        retval |= BuscaParesDesnudosFila(f);
    }
    for(int c=0; c<9; c++) {
        retval |= BuscaParesDesnudosColumna(c);
    }
    for(int k=0; k<9; k++) {
        retval |= BuscaParesDesnudosCaja(k);
    }

    // Eliminar elementos marcados para borrar:
    for(ListaDesnudo()->Primero(); ListaDesnudo()->Actual(); ListaDesnudo()->Siguiente())
        if(ListaDesnudo()->ValorActual().MarcadoParaBorrar()) ListaDesnudo()->BorrarActual();

    return retval;
}

bool Tablero::ProcesaParesDesnudosFila(int f) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, fil;
    char cad[100];

    for(int c=0; c<9; c++) {
        caslibre[c] = !casilla[f][c].Valor();
        if(!caslibre[c]) {
            presente[casilla[f][c].Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        fil = ListaDesnudo()->ValorActual().MismaFila();
        if(fil == f) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->Columna()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->Columna()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,c=0; c<9; c++) {
            if(caslibre[c]) libre[i++]=c;
        }
        // En cada casilla libre marcar a lpiz los valores no presentes
        int k0 = casilla[f][libre[0]].Caja();
        int k1 = casilla[f][libre[1]].Caja();

        if(!casilla[f][libre[0]].Valor()) {
            retval |= casilla[f][libre[0]].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f][libre[0]].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[f][libre[1]].Valor()) {
            retval |= casilla[f][libre[1]].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f][libre[1]].Lapiz().AsignaMarca(lapiz[1]);
        }

        if(columna[libre[0]].Presente(lapiz[0]) || caja[k0].Presente(lapiz[0])) {
            Asignar(f, libre[0], lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f, '1'+libre[0]);
            Log(cad);
            retval=true;
        } else
        if(columna[libre[0]].Presente(lapiz[1]) || caja[k0].Presente(lapiz[1])) {
            Asignar(f, libre[0], lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f, '1'+libre[0]);
            Log(cad);
            retval=true;
        }

        if(columna[libre[1]].Presente(lapiz[0]) || caja[k1].Presente(lapiz[0])) {
            Asignar(f, libre[1], lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f, '1'+libre[1]);
            Log(cad);
            retval=true;
        } else
        if(columna[libre[1]].Presente(lapiz[1]) || caja[k1].Presente(lapiz[1])) {
            Asignar(f, libre[1], lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f, '1'+libre[1]);
            Log(cad);
            retval=true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosColumna(int c) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, col;
    char cad[100];

    for(int f=0; f<9; f++) {
        caslibre[f] = !casilla[f][c].Valor();
        if(!caslibre[f]) {
            presente[casilla[f][c].Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        col = ListaDesnudo()->ValorActual().MismaColumna();
        if(col == c) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->Fila()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->Fila()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,f=0; f<9; f++) {
            if(caslibre[f]) libre[i++]=f;
        }
        // En cada casilla libre marcar a lpiz los valores no presentes
        int k0 = casilla[libre[0]][c].Caja();
        int k1 = casilla[libre[1]][c].Caja();

        if(!casilla[libre[0]][c].Valor()) {
            retval |= casilla[libre[0]][c].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[libre[0]][c].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[libre[1]][c].Valor()) {
            retval |= casilla[libre[1]][c].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[libre[1]][c].Lapiz().AsignaMarca(lapiz[1]);
        }

        if(fila[libre[0]].Presente(lapiz[0]) || caja[k0].Presente(lapiz[0])) {
            Asignar(libre[0], c, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+libre[0], '1'+c);
            Log(cad);
            retval=true;
        } else
        if(fila[libre[0]].Presente(lapiz[1]) || caja[k0].Presente(lapiz[1])) {
            Asignar(libre[0], c, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+libre[0], '1'+c);
            Log(cad);
            retval=true;
        }

        if(fila[libre[1]].Presente(lapiz[0]) || caja[k1].Presente(lapiz[0])) {
            Asignar(libre[1], c, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+libre[1], '1'+c);
            Log(cad);
            retval=true;
        } else
        if(fila[libre[1]].Presente(lapiz[1]) || caja[k1].Presente(lapiz[1])) {
            Asignar(libre[1], c, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+libre[1], '1'+c);
            Log(cad);
            retval=true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosCaja(int k) {
    bool retval = false;
    bool presente[9] = { false, false, false, false, false, false, false, false, false };
    bool caslibre[9];
    int lapiz[2] = { 0, 0 };
    int libre[2] = { 0, 0 };
    int libres=9, caj;
    char cad[100];

    for(int p=0; p<9; p++) {
        caslibre[p] = !caja[k].Cas(p)->Valor();
        if(!caslibre[p]) {
            presente[caja[k].Cas(p)->Valor()-1] = true;
            libres--;
        }
    }
    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        caj = ListaDesnudo()->ValorActual().MismaCaja();
        if(caj == k) {
            caslibre[ListaDesnudo()->ValorActual().Cas(0)->PosEnCaja()] = false;
            caslibre[ListaDesnudo()->ValorActual().Cas(1)->PosEnCaja()] = false;
            libres -= 2;
            presente[ListaDesnudo()->ValorActual().Val(0)-1] = true;
            presente[ListaDesnudo()->ValorActual().Val(1)-1] = true;
        }
        ListaDesnudo()->Siguiente();
    }
    if(libres==2) { // Nuevo par desnudo
        // Obtener los dos valores del nuevo par:
        for(int i=0,v=1; v<=9; v++) {
            if(!presente[v-1]) lapiz[i++]=v;
        }
        for(int i=0,p=0; p<9; p++) {
            if(caslibre[p]) libre[i++]=p;
        }
        int f0=caja[k].Cas(libre[0])->Fila();
        int f1=caja[k].Cas(libre[1])->Fila();
        int c0=caja[k].Cas(libre[0])->Columna();
        int c1=caja[k].Cas(libre[1])->Columna();

        if(!casilla[f0][c0].Valor()) {
            retval |= casilla[f0][c0].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f0][c0].Lapiz().AsignaMarca(lapiz[1]);
        }
        if(!casilla[f1][c1].Valor()) {
            retval |= casilla[f1][c1].Lapiz().AsignaMarca(lapiz[0]);
            retval |= casilla[f1][c1].Lapiz().AsignaMarca(lapiz[1]);
        }

        // En cada casilla libre marcar a lpiz los valores no presentes
        if(fila[f0].Presente(lapiz[0]) || columna[c0].Presente(lapiz[0])) {
            Asignar(f0, c0, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f0, '1'+c0);
            Log(cad);
            retval = true;
        } else
        if(fila[f0].Presente(lapiz[1]) || columna[c0].Presente(lapiz[1])) {
            Asignar(f0, c0, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f0, '1'+c0);
            Log(cad);
            retval = true;
        }
        if(fila[f1].Presente(lapiz[0]) || columna[c1].Presente(lapiz[0])) {
            Asignar(f1, c1, lapiz[1]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[1], 'A'+f1, '1'+c1);
            Log(cad);
            retval = true;
        } else
        if(fila[f1].Presente(lapiz[1]) || columna[c1].Presente(lapiz[1])) {
            Asignar(f1, c1, lapiz[0]);
            sprintf(cad, "Valor final %d en %c%c", lapiz[0], 'A'+f1, '1'+c1);
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudos() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaParesDesnudosFila(f);
    for(int c=0; c<9; c++) retval |= ProcesaParesDesnudosColumna(c);
    for(int k=0; k<9; k++) retval |= ProcesaParesDesnudosCaja(k);
    return retval;
}

// Procedimientos de fase 2:
void Tablero::PonerMarcasLapiz() {
    // Para cada casilla colocar marcas de lpiz que faltan:
    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(!casilla[f][c].Valor()) { // Si no tiene un valor asignado
                for(int v=1; v<=9; v++) {
                    int k = casilla[f][c].Caja();
                    if(!fila[f].Presente(v) && !columna[c].Presente(v) && !caja[k].Presente(v))
                        casilla[f][c].Lapiz().AsignaMarca(v);
                }
            }
        }
    }
    for(int v=1; v<=9; v++) {
        int ncaja=0;
        for(int f=0; f<3; f++) { // Fila de cajas
            for(int c=0; c<3; c++) { // Columna de cajas
                for(int i=0; i<3; i++) { // Fila de punteros
                    if(caja[ncaja].PunteroFila(i, v)) fila[f*3+i].DesmarcaLapizMenosCaja(c, v);
                }
                for(int i=0; i<3; i++) { // Columna de punteros
                    if(caja[ncaja].PunteroColumna(i, v)) columna[c*3+i].DesmarcaLapizMenosCaja(f, v);
                }
                ncaja++;
            }
        }
    }
}

bool Tablero::CasillasConUnaMarcaLapiz() {
    bool retval=false;
    int v;
    char cad[100];

    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() == 1) {
                retval = true;
                v=1;
                while(!casilla[f][c].Lapiz().ContieneMarca(v)) v++;
                Asignar(f, c, v);
                sprintf(cad, "Unica Marca Lapiz %c%c, valor %d", 'A'+f, '1'+c, v);
                Log(cad);
            }
        }
    }
    return retval;
}

bool Tablero::UnicaCasillaConCandidato() {
    bool retval = false;
    Unico u(0,0);
    char cad[100];

    // Buscar casillas con nico valor de candidato en filas, columnas y cajas:
    for(int f=0; f<9; f++) {
        u = fila[f].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            Asignar(f, u.Casilla(), u.Valor());
            sprintf(cad, "Unica Casilla Candidato Fila %c%c, valor %d", 'A'+f, '1'+u.Casilla(), u.Valor());
            Log(cad);
            retval = true;
        }
    }

    for(int c=0; c<9; c++) {
        u = columna[c].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            Asignar(u.Casilla(), c, u.Valor());
            sprintf(cad, "Unica Casilla Candidato Columna %c%c, valor %d", 'A'+u.Casilla(), '1'+c, u.Valor());
            Log(cad);
            retval = true;
        }
    }

    for(int k=0; k<9; k++) {
        u = caja[k].UnicaCasillaConCandidato();
        if(u.Casilla() != -1) {
            int fil = caja[k].Cas(u.Casilla())->Fila();
            int col = caja[k].Cas(u.Casilla())->Columna();
            Asignar(fil, col, u.Valor());
            sprintf(cad, "Unica Casilla Candidato Caja %d %c%c, valor %d", k, 'A'+fil, '1'+col, u.Valor());
            Log(cad);
            retval = true;
        }
    }
    return retval;
}

bool Tablero::VerificarFinales() {
    bool retval=false;

    while(CasillasConUnaMarcaLapiz()) {
        Texto(5,40,"CasillasConUnaMarcaLapiz",7);
        retval = true;
    }
    while(UnicaCasillaConCandidato()) {
        Texto(6,40,"UnicaCasillaConCandidato",7);
        retval = true;
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosFilaFase2(int f) {
    bool retval = false;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int fil = ListaDesnudo()->ValorActual().MismaFila();
        if(fil == f) {
            for(int c=0; c<9; c++) {
                if(c != ListaDesnudo()->ValorActual().Cas(0)->Columna() &&
                   c != ListaDesnudo()->ValorActual().Cas(1)->Columna() &&
                   (!ListaDesnudo()->ValorActual().Trio() || c!= ListaDesnudo()->ValorActual().Cas(2)->Columna())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    };
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosColumnaFase2(int c) {
    bool retval = false;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int col = ListaDesnudo()->ValorActual().MismaColumna();
        if(col == c) {
            for(int f=0; f<9; f++) {
                if(f != ListaDesnudo()->ValorActual().Cas(0)->Fila() &&
                   f != ListaDesnudo()->ValorActual().Cas(1)->Fila()&&
                   (!ListaDesnudo()->ValorActual().Trio() || f!= ListaDesnudo()->ValorActual().Cas(2)->Fila())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    }
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosCajaFase2(int k) {
    bool retval = false;
    int f, c;
    char cad[100];

    // Procesar pares desnudos
    ListaDesnudo()->Primero();
    while(ListaDesnudo()->Actual()) {
        int caj = ListaDesnudo()->ValorActual().MismaCaja();
        if(caj == k) {
            for(int p=0; p<9; p++) {
                if(p != ListaDesnudo()->ValorActual().Cas(0)->PosEnCaja() &&
                   p != ListaDesnudo()->ValorActual().Cas(1)->PosEnCaja() &&
                   (!ListaDesnudo()->ValorActual().Trio() || p!= ListaDesnudo()->ValorActual().Cas(2)->PosEnCaja())) {
                    f = caja[k].Cas(p)->Fila();
                    c = caja[k].Cas(p)->Columna();
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(0))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(0), 'A'+f, '1'+c);
                        Log(cad);
                    }
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaDesnudo()->ValorActual().Val(1))) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaDesnudo()->ValorActual().Val(1), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaDesnudo()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaParesDesnudosFase2() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaParesDesnudosFilaFase2(f);
    for(int c=0; c<9; c++) retval |= ProcesaParesDesnudosColumnaFase2(c);
    for(int k=0; k<9; k++) retval |= ProcesaParesDesnudosCajaFase2(k);
    return retval;
}

// Buscar pares y trios puntero
bool Tablero::BuscaPunteros() {
    bool retval = false;

    for(int v=1; v<=9; v++) {
        for(int f=0; f<9; f++)
            retval |= BuscaPunterosFila(f, v);
        for(int c=0; c<9; c++)
            retval |= BuscaPunterosColumna(c, v);
        for(int k=0; k<9; k++)
            retval |= BuscaPunterosCaja(k, v);
    }
    return retval;
}

// Busca el candidato v en la fila f, si est dos o tres veces en la misma caja lo guarda.
bool Tablero::BuscaPunterosFila(int f, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!fila[f].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int c=0; c<9; c++) if(fila[f].Cas(c)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int c=0; c<9; c++) {
                if(fila[f].Cas(c)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, fila[f].Cas(c));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaCaja()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::BuscaPunterosColumna(int c, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!columna[c].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int f=0; f<9; f++) if(columna[c].Cas(f)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int f=0; f<9; f++) {
                if(columna[c].Cas(f)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, columna[c].Cas(f));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaCaja()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::BuscaPunterosCaja(int k, int v) {
    int cuenta = 0;

    // Si el valor v est asignado a la fila, retornar false
    if(!caja[k].Presente(v)) {
        // Leer casillas con la marca de lpiz v. Si son ms de tres retornar false
        for(int p=0; p<9; p++) if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) cuenta++;
        if(cuenta == 3 || cuenta == 2) {
            Puntero pun(cuenta);
            cuenta = 0;
            pun.AsignarVal(v);
            for(int p=0; p<9; p++) {
                if(caja[k].Cas(p)->Lapiz().ContieneMarca(v)) {
                    pun.AsignarCas(cuenta++, caja[k].Cas(p));
                }
            }
            // Si todas las casillas estn en la misma caja, almacenar.
            if(pun.MismaFila()!=-1 || pun.MismaColumna()!=-1) {
                puntero.Insertar(pun);
                return true;
            }
        }
    }
    return false;
}

bool Tablero::ProcesaPunterosFila(int f) {
    bool retval = false;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int fil = ListaPuntero()->ValorActual().MismaFila();
        if(fil == f) {
            for(int c=0; c<9; c++) { // Comparar casillas?? casilla[f][c] != ListaPuntero()->ValorActual().Cas(0)
                if(c != ListaPuntero()->ValorActual().Cas(0)->Columna() &&
                   c != ListaPuntero()->ValorActual().Cas(1)->Columna() &&
                   (!ListaPuntero()->ValorActual().Trio() || c!= ListaPuntero()->ValorActual().Cas(2)->Columna())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunterosColumna(int c) {
    bool retval = false;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int col = ListaPuntero()->ValorActual().MismaColumna();
        if(col == c) {
            for(int f=0; f<9; f++) {
                if(f != ListaPuntero()->ValorActual().Cas(0)->Fila() &&
                   f != ListaPuntero()->ValorActual().Cas(1)->Fila()&&
                   (!ListaPuntero()->ValorActual().Trio() || f!= ListaPuntero()->ValorActual().Cas(2)->Fila())) {
                    if(casilla[f][c].Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunterosCaja(int k) {
    bool retval = false;
    int f, c;
    char cad[100];

    // Procesar punteros
    ListaPuntero()->Primero();
    while(ListaPuntero()->Actual()) {
        int caj = ListaPuntero()->ValorActual().MismaCaja();
        if(caj == k) {
            for(int p=0; p<9; p++) {
                if(p != ListaPuntero()->ValorActual().Cas(0)->PosEnCaja() &&
                   p != ListaPuntero()->ValorActual().Cas(1)->PosEnCaja() &&
                   (!ListaPuntero()->ValorActual().Trio() || p!= ListaPuntero()->ValorActual().Cas(2)->PosEnCaja())) {
                    if(caja[k].Cas(p)->Lapiz().EliminaMarca(ListaPuntero()->ValorActual().Val())) {
                        retval = true;
                        f = caja[k].Cas(p)->Fila();
                        c = caja[k].Cas(p)->Columna();
                        sprintf(cad, "Eliminar candidato %d en %c%c", ListaPuntero()->ValorActual().Val(), 'A'+f, '1'+c);
                        Log(cad);
                    }
                }
            }
        }
        ListaPuntero()->Siguiente();
    }
    return retval;
}

bool Tablero::ProcesaPunteros() {
    bool retval = false;

    for(int f=0; f<9; f++) retval |= ProcesaPunterosFila(f);
    for(int c=0; c<9; c++) retval |= ProcesaPunterosColumna(c);
    for(int k=0; k<9; k++) retval |= ProcesaPunterosCaja(k);
    return retval;
}

bool Tablero::ProcesaGrafoFila(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    // Para cada fila:
    for(int f=0; f<9; f++) {
        g.IniciarMarcas(fila[f].GetCasillas());
        g.AsignarAristas();
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de fila:
            for(int c=0; c<9; c++) {
                if(!g.Contiene(c)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= fila[f].Cas(c)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int c=0; c<9; c++) {
                    if(!g.Contiene(c)) {
                        for(int i=1; i<=grupo; i++) {
                            if(fila[f].Cas(c)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+f, '1'+c);
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafoColumna(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    for(int c=0; c<9; c++) {
        g.IniciarMarcas(columna[c].GetCasillas());
        g.AsignarAristas();
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de columna:
            for(int f=0; f<9; f++) {
                if(!g.Contiene(f)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= columna[c].Cas(f)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int f=0; f<9; f++) {
                    if(!g.Contiene(f)) {
                        for(int i=1; i<=grupo; i++) {
                            if(columna[c].Cas(f)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+f, '1'+c);
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafoCaja(int grupo) {
    Grafo g(grupo);
    bool retval = false;
    char cad[100];

    for(int k=0; k<9; k++) {
        g.IniciarMarcas(caja[k].GetCasillas());
        g.AsignarAristas();
        // Para cada grupo existente
        while(g.BuscarGrupo()) {
            // Eliminar candidatos en resto de caja:
            for(int p=0; p<9; p++) {
                if(!g.Contiene(p)) {
                    for(int i=1; i<=grupo; i++) {
                        retval |= caja[k].Cas(p)->Lapiz().ContieneMarca(g.Valor(i));
                    }
                }
            }
            if(retval) {
                sprintf(cad, "Grupo[%d] cas: %s marcas: %s", grupo, g.Casillas(), g.Valores());
                Log(cad);
                for(int p=0; p<9; p++) {
                    if(!g.Contiene(p)) {
                        for(int i=1; i<=grupo; i++) {
                            if(caja[k].Cas(p)->Lapiz().EliminaMarca(g.Valor(i))) {
                                sprintf(cad, "Eliminar marca %d en %c%c", g.Valor(i), 'A'+caja[k].Cas(p)->Fila(), '1'+caja[k].Cas(p)->Columna());
                                Log(cad);
                            }
                        }
                    }
                }
            }
            g.EliminaGrupo();
        }
    }
    return retval;
}

bool Tablero::ProcesaGrafo(int grupo) {
    bool retval = false;

    retval |= ProcesaGrafoFila(grupo);
    retval |= ProcesaGrafoColumna(grupo);
    retval |= ProcesaGrafoCaja(grupo);

    return retval;
}

bool Tablero::ProcesaXYWing() {
    int f, c;
    Casilla *pivote;
    Casilla *ala1;
    Casilla *ala2;
    int c1, f1;

    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(casilla[f][c].Lapiz().CuentaMarcas() == 2) { // Posible pivote
                pivote = &casilla[f][c];
                c1 = 0;
                do {
                    ala1 = fila[f].BuscaAla1MenosCaja(pivote, c1);
                    if(ala1) { // Tenemos posible ala 1 en f,c2 Casos 1/2
                        c1 = ala1->Columna()+1; // Probar si hay ms candidatos a ala1
                        // Buscar ala2
                        ala2 = columna[c].BuscaAla2MenosCaja(pivote, ala1);
                        if(ala2) { // Patrn XY-Wing caso 1
                            if(ProcesarXYWingCaso1(pivote, ala1, ala2)) return true;
                        } else { // Posible caso 2
                            ala2 = caja[pivote->Caja()].BuscaAla2MenosFila(pivote, ala1);
                            if(ala2) { // Patrn XY-Wing caso 2
                                if(ProcesarXYWingCaso2(pivote, ala1, ala2)) return true;
                            }
                        }
                    } else { // Posible caso 3
                        c1 = 9; // No hemos encontrado el ala1 en la fila f. No seguir buscando.
                        f1 = 0;
                        do {
                            ala1 = columna[c].BuscaAla1MenosCaja(pivote, f1);
                            if(ala1) { // Tenemos posible ala 1 en f2,c
                                f1 = ala1->Fila()+1;
                                ala2 = caja[pivote->Caja()].BuscaAla2MenosColumna(pivote, ala1);
                                if(ala2) { // Patrn XY-Wing caso 3
                                    if(ProcesarXYWingCaso3(pivote, ala1, ala2)) return true;
                                }
                            }
                        } while(ala1 && f1 < 9); // Probar si hay ms candidatos a ala1
                    }
                } while(ala1 && c1 < 9);
            }
        }
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso1(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    char cad[100];

    if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 1 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala2->Fila(), '1'+ala1->Columna());
            Log(cad);
            casilla[ala2->Fila()][ala1->Columna()].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso2(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    int c1, c2, c3;
    int k = pivote->Caja();
    char cad[100];

    c1 = caja[k].Cas(0)->Columna();
    c2 = caja[k].Cas(1)->Columna();
    c3 = caja[k].Cas(2)->Columna();
    if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c1].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c2].Lapiz().ContieneMarca(valor) ||
       casilla[pivote->Fila()][c3].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 2 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala2->Fila()][ala1->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala2->Fila(), '1'+ala1->Columna());
            Log(cad);
            casilla[ala2->Fila()][ala1->Columna()].Resaltar();
        }
        if(casilla[pivote->Fila()][c1].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c1);
            Log(cad);
            casilla[pivote->Fila()][c1].Resaltar();
        }
        if(casilla[pivote->Fila()][c2].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c2);
            Log(cad);
            casilla[pivote->Fila()][c2].Resaltar();
        }
        if(casilla[pivote->Fila()][c3].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+pivote->Fila(), '1'+c3);
            Log(cad);
            casilla[pivote->Fila()][c3].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesarXYWingCaso3(Casilla *pivote, Casilla *ala1, Casilla *ala2) {
    MarcasLapiz comun = ala1->Lapiz() ^ ala2->Lapiz();
    int valor = comun.Valor();
    int f1, f2, f3;
    int k = pivote->Caja();
    char cad[100];

    f1 = caja[k].Cas(0)->Fila();
    f2 = caja[k].Cas(3)->Fila();
    f3 = caja[k].Cas(6)->Fila();
    if(casilla[ala1->Fila()][ala2->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f1][pivote->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f2][pivote->Columna()].Lapiz().ContieneMarca(valor) ||
       casilla[f3][pivote->Columna()].Lapiz().ContieneMarca(valor)) {
        LimpiaResaltes();
        sprintf(cad, "XY-Wing caso 3 %c%c -> %c%c %c%c [%c]",
            'A'+pivote->Fila(), '1'+pivote->Columna(),
            'A'+ala1->Fila(), '1'+ala1->Columna(),
            'A'+ala2->Fila(), '1'+ala2->Columna(), '0'+valor);
        Log(cad);
        pivote->Amarillo();
        ala1->Naranja();
        ala2->Naranja();
        if(casilla[ala1->Fila()][ala2->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+ala1->Fila(), '1'+ala2->Columna());
            Log(cad);
            casilla[ala1->Fila()][ala2->Columna()].Resaltar();
        }
        if(casilla[f1][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f1, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        if(casilla[f2][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f2, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        if(casilla[f3][pivote->Columna()].Lapiz().EliminaMarca(valor)) {
            sprintf(cad, "-> Elimina %c de %c%c", '0'+valor, 'A'+f3, '1'+pivote->Columna());
            Log(cad);
            casilla[f1][pivote->Columna()].Resaltar();
        }
        return true;
    }
    return false;
}

bool Tablero::ProcesaXWing() {
    Pareja par1, par2;

    LimpiaResaltes();

    for(int v=1; v<=9; v++) { // Para cada valor
        // Empezamos con las filas. No buscaremos en la ltima,
        // ya que no podemos comparar con las siguientes
        for(int f=0; f<8; f++) {
            // Detectar fila con dos casillas con el candidato v
            par1 = fila[f].DobleCandidato(v);
            // Si par1(0) != NULL, tenemos un candidato a X-Wing, buscaremos en el resto de filas
            if(par1.Cas(0) != NULL) {
                for(int f2=f+1; f2<9; f2++) {
                    par2 = fila[f2].DobleCandidato(v);
                    if(par2.Cas(0) != NULL &&
                       par1.Cas(0)->Columna() == par2.Cas(0)->Columna() &&
                       par1.Cas(1)->Columna() == par2.Cas(1)->Columna()) {
                        if(ProcesaXWingFila(par1, par2, v)) return true;
                    }
                }
            }
        }
        // Y ahora las columnas.
        for(int c=0; c<8; c++) {
            // Detectar fila con dos casillas con el candidato v
            par1 = columna[c].DobleCandidato(v);
            // Si par1(0) != NULL, tenemos un candidato a X-Wing, buscaremos en el resto de columnas
            if(par1.Cas(0) != NULL) {
                for(int c2=c+1; c2<9; c2++) {
                    par2 = columna[c2].DobleCandidato(v);
                    if(par2.Cas(0) != NULL &&
                       par1.Cas(0)->Fila() == par2.Cas(0)->Fila() &&
                       par1.Cas(1)->Fila() == par2.Cas(1)->Fila()) {
                        if(ProcesaXWingColumna(par1, par2, v)) return true;
                    }
                }
            }
        }
    }
    return false;
}

bool Tablero::ProcesaXWingFila(Pareja a, Pareja b, int v) {
    bool retval=false;
    char cad[100];

    for(int f=0; f<9; f++) {
        if(f != a.Cas(0)->Fila() && f != b.Cas(0)->Fila()) {
            retval |= casilla[f][a.Cas(0)->Columna()].Lapiz().ContieneMarca(v);
            retval |= casilla[f][a.Cas(1)->Columna()].Lapiz().ContieneMarca(v);
        }
    }
    if(retval) {
        sprintf(cad, "X-Wing %c%c %c%c %c%c %c%c valor %d",
            'A'+a.Cas(0)->Fila(), '1'+a.Cas(0)->Columna(),
            'A'+a.Cas(1)->Fila(), '1'+a.Cas(1)->Columna(),
            'A'+b.Cas(0)->Fila(), '1'+b.Cas(0)->Columna(),
            'A'+b.Cas(1)->Fila(), '1'+b.Cas(1)->Columna(), v);
        Log(cad);
        a.Cas(0)->Amarillo();
        a.Cas(1)->Amarillo();
        b.Cas(0)->Amarillo();
        b.Cas(1)->Amarillo();
        for(int f=0; f<9; f++) {
            if(f != a.Cas(0)->Fila() && f != b.Cas(0)->Fila()) {
                if(casilla[f][a.Cas(0)->Columna()].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+f, '1'+a.Cas(0)->Columna());
                    Log(cad);
                    casilla[f][a.Cas(0)->Columna()].Resaltar();
                }
                if(casilla[f][a.Cas(1)->Columna()].Lapiz().EliminaMarca(v)) {
                    sprintf(cad, " -> Elimina candidato %d de %c%c", v, 'A'+f, '1'+a.Cas(1)->Columna());
                    Log(cad);
                    casilla[f][a.Cas(1)->Columna()].Resaltar();
                }
            }
        }
    }

    return retval;
}

bool Tablero::ProcesaXWingColumna(Pareja a, Pareja b, int v) {
    bool retval=false;

    for(int c=0; c<9; c++) {
        if(c != a.Cas(0)->Columna() && c != b.Cas(0)->Columna()) {
            retval |= casilla[a.Cas(0)->Fila()][c].Lapiz().EliminaMarca(v);
            retval |= casilla[a.Cas(1)->Fila()][c].Lapiz().EliminaMarca(v);
        }
    }
    return retval;
}

void Tablero::Mostrar() {
    int f, c;
    WORD color1, color2;

    color1 = 15;
    color2 = 3;

    for(f=0; f<9; f++) {
        for(c=0; c<9; c++) {
            if(f%3==0) {
                Texto(f*4,  c*4, "+", color1);
                Texto(f*4,  c*4+1, "---+", color1);
            } else {
                Texto(f*4,  c*4, "+", color2);
                Texto(f*4,  c*4+1, "---+", color2);
            }
            if(c%3==0) {
                Texto(f*4+1,c*4, "|", color1);
                Texto(f*4+2,c*4, "|", color1);
                Texto(f*4+3,c*4, "|", color1);
            } else {
                Texto(f*4+1,c*4, "|", color2);
                Texto(f*4+1,c*4+4, "|", color2);
                Texto(f*4+2,c*4, "|", color2);
                Texto(f*4+2,c*4+4, "|", color2);
                Texto(f*4+3,c*4, "|", color2);
                Texto(f*4+3,c*4+4, "|", color2);
            }
            Texto(f*4+1,c*4+4, "|", color1);
            Texto(f*4+2,c*4+4, "|", color1);
            Texto(f*4+3,c*4+4, "|", color1);
        }
    }
    for(c=0; c<9; c++) {
        Texto(36,  c*4, "+", color1);
        Texto(36,  c*4+1, "---+", color1);
    }

    for(int f=0; f<9; f++) {
        for(int c=0; c<9; c++) {
            casilla[f][c].Mostrar();
        }
    }
    Texto(55, 0, " ", color1);
}

