// Sudokus con Clase
// Resolucin de sudokus con tcnicas reales
// (C) 2022 Salvador Pozo
// Con Clase https://conclase.net
//
// Declaracin de la clase Tablero

#ifndef SUDOKU_TABLERO
#define SUDOKU_TABLERO

#include <fstream>
#include "casilla.h"
#include "fila.h"
#include "columna.h"
#include "caja.h"
#include "lista.h"
#include "desnudo.h"
#include "puntero.h"
#include "grafo.h"

class Tablero {
    private:
    Casilla casilla[9][9];
    Fila fila[9];
    Columna columna[9];
    Caja caja[9];
    int cuenta;
    std::ofstream flog;
    Lista<Desnudo> desnudo;
    Lista<Puntero> puntero;

    bool BuscaParesDesnudosFila(int f);
    bool BuscaParesDesnudosColumna(int c);
    bool BuscaParesDesnudosCaja(int c);

    bool ProcesaParesDesnudosFila(int f);
    bool ProcesaParesDesnudosColumna(int c);
    bool ProcesaParesDesnudosCaja(int c);

    bool ProcesaParesDesnudosFilaFase2(int f);
    bool ProcesaParesDesnudosColumnaFase2(int c);
    bool ProcesaParesDesnudosCajaFase2(int c);

    bool BuscaPunterosFila(int f, int v);
    bool BuscaPunterosColumna(int c, int v);
    bool BuscaPunterosCaja(int k, int v);

    bool ProcesaPunterosFila(int f);
    bool ProcesaPunterosColumna(int c);
    bool ProcesaPunterosCaja(int k);

    bool ProcesaGrafoFila(int grupo);
    bool ProcesaGrafoColumna(int grupo);
    bool ProcesaGrafoCaja(int grupo);

    bool ProcesarXYWingCaso1(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso2(Casilla *pivote, Casilla *ala1, Casilla *ala2);
    bool ProcesarXYWingCaso3(Casilla *pivote, Casilla *ala1, Casilla *ala2);

    bool ProcesaXWingFila(Pareja a, Pareja b, int v);
    bool ProcesaXWingColumna(Pareja a, Pareja b, int v);

    public:
    Tablero(char* t);
    ~Tablero() { flog.close(); }
    void Log(char *cad);
    Casilla &GetCasilla(int f, int c) { return casilla[f][c]; }
    const int Cuenta() { return cuenta; }
    void Mostrar();
    bool Soluciones();
    bool Valido(int f, int c, int v);
    void Asignar(int f, int c, int v);

    // Resolucin:
    void LimpiaResaltes();
    void LimpiaCriba();
    void Criba(int v);
    bool ProcesarCriba(int v);
    bool CandidatoUnico();
    bool ProcesarCribaSolitarios(int v);
    bool BuscaParesDesnudos();
    bool ProcesaParesDesnudos();
    // Resolucion FASE 2
    void PonerMarcasLapiz();
    bool CasillasConUnaMarcaLapiz();
    bool UnicaCasillaConCandidato();
    bool VerificarFinales();
    bool ProcesaParesDesnudosFase2();
    bool BuscaPunteros();
    bool ProcesaPunteros();
    bool ProcesaGrafo(int grupo);
    bool ProcesaXYWing();
    bool ProcesaXWing();

    Lista<Desnudo> *ListaDesnudo() { return &desnudo; }
    Lista<Puntero> *ListaPuntero() { return &puntero; }
};
#endif // SUDOKU_TABLERO
